from collective.transmogrifier.interfaces import ISection
from collective.transmogrifier.interfaces import ISectionBlueprint
from collective.transmogrifier.utils import Condition
from collective.transmogrifier.utils import Expression
from collective.transmogrifier.utils import defaultMatcher
from zope.interface import classProvides, implements
import re


class RegexReplacer(object):
    classProvides(ISectionBlueprint)
    implements(ISection)

    """Replaces strings with regex:

    [insert-table-class]
    blueprint = ftw.blueprints.regexreplacer
    key = string:text
    pattern = python: r'<table[^>]*>'
    repl = python: '<table class="listing">'
    condition = python: item.get('_type') == 'TextBlock'
    """
    def __init__(self, transmogrifier, name, options, previous):
        self.previous = previous
        self.key = Expression(options['key'], transmogrifier, name, options)
        self.pattern = Expression(
            options['pattern'], transmogrifier, name, options)
        self.repl = Expression(options['repl'], transmogrifier, name, options)
        self.condition = Condition(options.get('condition', 'python:True'),
                                   transmogrifier, name, options)

    def __iter__(self):
        for item in self.previous:
            if self.condition(item):
                key = self.key(item)
                if item.get(key):
                    item[key] = re.sub(
                        self.pattern(item), self.repl(item), item.get(key))

            yield item


class FieldMapper(object):
    classProvides(ISectionBlueprint)
    implements(ISection)

    def __init__(self, transmogrifier, name, options, previous):
        """The FieldMapper Blueprint provides powerful functionality to map and
        modify values on the given item.
        """
        self.mapper = Expression(
            options['field-mapping'], transmogrifier, name, options)

        self.condition = Condition(options.get('condition', 'python:True'),
            transmogrifier, name, options)

        self.previous = previous

    def __iter__(self):
        for item in self.previous:

            if not self.condition(item):
                yield item
                continue

            mapper = self.mapper(item)
            for src, dest in mapper.items():
                dest_name = dest.get('destination', None)
                transform_value = dest.get('transform', '')
                static_value = dest.get('static_value', '')
                map_value = dest.get('map_value', '')
                src_val = item.get(src, None)
                need_src_key = dest.get('need_src_key', False)

                if need_src_key and src_val is None:
                    continue

                if not dest_name:
                    dest_name = src

                if static_value:
                    item[dest_name] = static_value
                    continue

                if transform_value:
                    if callable(transform_value):
                        src_val = transform_value(item)

                if map_value:
                    src_val = map_value.get(src_val, src_val)

                item[dest_name] = src_val

            yield item


class PathMapper(object):
    """Maps old paths to new paths.

    Applices recursive mapping if the path at pathkey is a dict. Applies
    mapping for each element if path at pathkey is another iterable.

    All prefixes configured in strip_prefixes will be removed from the
    path.

    """
    classProvides(ISectionBlueprint)
    implements(ISection)

    def __init__(self, transmogrifier, name, options, previous):
        self.previous = previous
        self.condition = Condition(options.get('condition', 'python:True'),
                                   transmogrifier, name, options)
        self.strip_prefixes = Expression(options.get('strip-prefixes',
                                                     'python: []'),
                                         transmogrifier, name, options)
        self.mapping = Expression(options['mapping'],
                                  transmogrifier, name, options)
        self.pathkey = defaultMatcher(options, 'path-key', name, 'path')

    def _apply_mapping(self, strip_prefixes, mapping, path):
        for each in strip_prefixes:
            if path.startswith(each):
                path = path.replace(each, '', 1)
        for pattern, repl in mapping:
            path = re.sub(pattern, repl, path)
        return path

    def _apply_mapping_recursively(self, strip_prefixes, mapping,
                                   item, pathkey, path):
        if isinstance(path, dict):
            for key, value in path.items():
                self._apply_mapping_recursively(strip_prefixes, mapping,
                                                path, key, value)
        elif hasattr(path, '__iter__'):
            item[pathkey] = list(self._apply_mapping(strip_prefixes,
                                                     mapping, each)
                                 for each in path)
        else:
            item[pathkey] = self._apply_mapping(strip_prefixes, mapping, path)

    def __iter__(self):
        for item in self.previous:
            keys = item.keys()
            pathkey = self.pathkey(*keys)[0]

            if not pathkey:
                yield item
                continue

            path = item[pathkey]
            if not self.condition(item, key=path):
                yield item
                continue

            mapping = self.mapping(item)
            strip_prefixes = self.strip_prefixes(item)
            self._apply_mapping_recursively(strip_prefixes, mapping,
                                            item, pathkey, path)
            yield item


class TypeFieldMapper(object):
    """Map types and their fields to new types and new fields.
    """

    classProvides(ISectionBlueprint)
    implements(ISection)

    def __init__(self, transmogrifier, name, options, previous):
        self.previous = previous
        self.mapping = Expression(options['mapping'],
                                  transmogrifier, name, options)
        self.typekey = defaultMatcher(options, 'type-key', name, 'type')

    def __iter__(self):
        for item in self.previous:
            keys = item.keys()
            typekey = self.typekey(*keys)[0]

            if not typekey:
                yield item
                continue

            old_type = item[typekey]
            type_mappings = self.mapping(item)
            if old_type in type_mappings:
                new_type, field_mappings = type_mappings[old_type]
                item[typekey] = new_type

                for old_field, new_field in field_mappings.items():
                    if old_field in keys:
                        item[new_field] = item[old_field]
                        if new_field != old_field:
                            del item[old_field]

            yield item



from ftw.blueprints.sections import mapper
from ftw.blueprints.tests.base import BlueprintTestCase


INPUT = {
    '_path': '/foo/bar',
    '_type': 'Folder',
    '_id': 'bar',
    'title': 'test',
    }


class TestFieldMapper(BlueprintTestCase):

    def setUp(self):
        self.klass = mapper.FieldMapper
        self.input_data = INPUT

    def test_do_no_changes(self):
        expected = self._get_expected()
        self.assert_result({'field-mapping': "python:{}"}, expected)

    def test_change_destination_name(self):
        expected = self._get_expected({'new-title': INPUT.get('title')})

        options = {
            'field-mapping': "python:{'title' :{'destination': 'new-title'}}"}

        self.assert_result(options, expected)

    def test_transform_value(self):
        expected = self._get_expected({'_id': 'test1'})

        options = {'field-mapping':
            "python:{'_id': {'transform': lambda x: '%s1' % x['title']}}"
            }

        self.assert_result(options, expected)

    def test_add_static_value(self):
        expected = self._get_expected({'_id': 'static_id'})

        options = {'field-mapping':
            "python:{'_id': {'static_value': 'static_id'}}"
            }

        self.assert_result(options, expected)

    def test_map_value(self):
        expected = self._get_expected({'title': 'mapped_title'})

        options = {'field-mapping':
            "python:{'title': {'map_value': {'test': 'mapped_title'}}}"
            }

        self.assert_result(options, expected)

    def test_add_new_item_without_src_key(self):
        expected = self._get_expected({'james': 'bond'})

        options = {'field-mapping':
            "python:{'xxx': {'destination': 'james', 'static_value': 'bond',}}"
            }

        self.assert_result(options, expected)

    def test_no_changes_on_item_when_need_src_key(self):
        expected = self._get_expected()

        options = {'field-mapping':
            "python:{'xxx': {'destination': 'james', " + \
            "'static_value': 'bond', 'need_src_key': True}}"
            }

        self.assert_result(options, expected)

    def test_change_destination_do_transform_and_map_value_on_same_item(self):
        expected = self._get_expected({'james': 'bond'})

        options = {'field-mapping':
            "python:{'xxx': {'destination': 'james', " + \
            "'transform': lambda x: x['_id'], 'map_value': {'bar': 'bond'}}}"
            }

        self.assert_result(options, expected)

    def test_multiple_changes_on_item(self):
        expected = self._get_expected({
            'title': 'static_title',
            '_id': 'bond',
            'reference': INPUT.get('_path'),
            })

        options = {'field-mapping':
            "python:{'title': {'static_value': 'static_title'}, " + \
            "'_id': {'map_value': {'bar': 'bond'}}, " + \
            "'_path': {'destination': 'reference'}}"
            }

        self.assert_result(options, expected)

    def test_do_no_changes_if_condition_is_false(self):
        expected = self._get_expected()

        options = {
            'field-mapping':
            "python:{'_id': {'static_value': 'static_id'}}",
            'condition': 'python:False'
            }

        self.assert_result(options, expected)



from collective.transmogrifier.interfaces import ISection
from collective.transmogrifier.interfaces import ISectionBlueprint
from collective.transmogrifier.utils import Condition
from collective.transmogrifier.utils import Expression
from collective.transmogrifier.utils import traverse
from importlib import import_module
from plone.portlets.interfaces import IPortletAssignmentMapping
from plone.portlets.interfaces import IPortletManager
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.interface import classProvides, implements
import logging


class PortletHandler(object):

    def __init__(self, manager_name, assignment_path, portlet_id):
        self.manager_name = manager_name
        self.portlet_id = portlet_id
        self.assignment_path = assignment_path

    def __call__(self, context, properties=None):
        properties = properties or {}
        manager = getUtility(IPortletManager, name=self.manager_name)
        assignment_mapping = getMultiAdapter(
            (context, manager), IPortletAssignmentMapping)

        # Set new portlet assignment
        if self.portlet_id in assignment_mapping:
            return

        assignment_mapping[self.portlet_id] = self.get_assignment_object(
            properties)

    def get_assignment_object(self, properties=None):
        """Return the assignment object of the defined portlet
        """
        properties = properties or {}
        assignment_class = self._get_assignment_class(self.assignment_path)

        return assignment_class(**properties)

    def _get_assignment_class(self, assignment_path):
        """Try to import and return the defined assignment class
        """
        path_elements = assignment_path.split('.')

        module = import_module('.'.join(path_elements[:-1]))
        assignment_class_name = path_elements[-1]

        return getattr(module, assignment_class_name)


class ContextualPortletAdder(object):
    classProvides(ISectionBlueprint)
    implements(ISection)

    def __init__(self, transmogrifier, name, options, previous):
        self.previous = previous
        self.context = transmogrifier.context
        self.logger = logging.getLogger(options['blueprint'])

        self.pathkey = options.get('path-key', '_path')

        self.portlet_properties = Expression(
            options.get('portlet-properties', 'python:{}'),
            transmogrifier, name, options)

        self.condition = Condition(options.get('condition', 'python:True'),
            transmogrifier, name, options)

        self.portlet_handler = PortletHandler(
            options.get('manager-name'),
            options.get('assignment-path'),
            options.get('portlet-id')
        )

    def __iter__(self):
        for item in self.previous:
            if not self.condition(item):
                yield item
                continue

            obj = traverse(self.context, item.get(self.pathkey, ''))

            if not obj:
                self.logger.warn(
                    "Context does not exist at %s" % item.get[self.pathkey])
                yield item
                continue

            self.portlet_handler(obj, self.portlet_properties(item))

            self.logger.info(
                "Added portlet at %s" % (item[self.pathkey]))



from ftw.blueprints.sections import mapper
from ftw.blueprints.tests.base import BlueprintTestCase


INPUT = {
    '_path': '/foo/bar',
    '_type': 'Folder',
    '_id': 'bar',
    'title': 'test',
    }


class TestPathMapper(BlueprintTestCase):

    def setUp(self):
        self.klass = mapper.PathMapper
        self.input_data = INPUT

    def _get_options(self, additional_options=None):
        options = {'blueprint': 'ftw.blueprints.pathmapper'}
        if additional_options:
            options.update(additional_options)
        return options

    def test_map_value(self):
        expected = self._get_expected({'_path': '/qux/bar'})

        options = self._get_options(
            {'mapping': "python:( ('^/foo', '/qux'), )"}
        )
        self.assert_result(options, expected)

    def test_strip_prefixes(self):
        input_data = self.input_data.copy()
        input_data['_path'] = '/nininini' + input_data['_path']
        expected = self._get_expected({'_path': '/qux/bar'})

        options = self._get_options(
            {'mapping': 'python:( ("^/foo", "/qux"), )',
             'strip-prefixes': 'python: ["/nininini"]'}
        )
        self.assert_result(options, expected, input_data=input_data)

    def test_recursive_strip_prefixes(self):
        input_data = self.input_data.copy()
        input_data['_refs'] = {'one': '/prefix/foo/bar',
                               'two': '/prefix/foo/qux'}

        expected = self._get_expected({'_refs': {'one': '/qux/bar',
                                                 'two': '/qux/qux'}})

        options = self._get_options({
            'mapping': "python:( ('^/foo', '/qux'), )",
            'strip-prefixes': 'python: ["/prefix"]',
            'path-key': '_refs',
        })
        self.assert_result(options, expected, input_data=input_data)

    def test_map_order(self):
        expected = self._get_expected({'_path': '/qux/bar'})

        options = self._get_options(
            {'mapping': "python:("
                "('^/foo', '/qux'),"
                "('^/foo/bar', '/nix/nax'),"
             ")"}
        )
        self.assert_result(options, expected)

    def test_map_condition(self):
        expected = self._get_expected()

        options = self._get_options({
            'mapping': "python:( ('^/foo', '/qux'), )",
            'condition': 'python: False',
        })
        self.assert_result(options, expected)

    def test_recursive_path_mapping_list(self):
        input_data = self.input_data.copy()
        input_data['_refs'] = ['/foo/bar', '/foo/qux']

        expected = self._get_expected({'_refs': ['/qux/bar', '/qux/qux']})

        options = self._get_options({
            'mapping': "python:( ('^/foo', '/qux'), )",
            'path-key': '_refs',
        })
        self.assert_result(options, expected, input_data=input_data)

    def test_recursive_path_mapping_dict(self):
        input_data = self.input_data.copy()
        input_data['_refs'] = {'one': '/foo/bar', 'two': '/foo/qux'}

        expected = self._get_expected({'_refs': {'one': '/qux/bar',
                                                 'two': '/qux/qux'}})

        options = self._get_options({
            'mapping': "python:( ('^/foo', '/qux'), )",
            'path-key': '_refs',
        })
        self.assert_result(options, expected, input_data=input_data)

    def test_recursive_path_mapping_nested_list_in_dict(self):
        input_data = self.input_data.copy()
        input_data['_refs'] = {'hehe': ['/foo/bar', '/foo/hehe'],
                               'two': '/foo/qux'}

        expected = self._get_expected({'_refs': {'hehe': ['/qux/bar',
                                                          '/qux/hehe'],
                                                 'two': '/qux/qux'}})

        options = self._get_options({
            'mapping': "python:( ('^/foo', '/qux'), )",
            'path-key': '_refs',
        })
        self.assert_result(options, expected, input_data=input_data)



from OFS.Image import File
from collective.transmogrifier.interfaces import ISection, ISectionBlueprint
from plone.app.transmogrifier.mimeencapsulator import MimeEncapsulatorSection
from zope.interface import classProvides, implements


class UnicodeAwareMimeEncapsulator(MimeEncapsulatorSection):
    """Make the default plone.app.transmogrifier.mimeencapsulator work with
    unicode fields.

    """
    classProvides(ISectionBlueprint)
    implements(ISection)

    def __iter__(self):
        for item in self.previous:
            key, match = self.datakey(*item.keys())
            if not key:
                yield item
                continue

            field = self.field(item, key=key, match=match)
            mimetype = self.mimetype(item, key=key, match=match, field=field)

            if self.condition(item,
                    key=key, match=match, field=field, mimetype=mimetype):
                value = item[key]
                if isinstance(value, unicode):
                    value = value.encode('utf-8')
                item[field] = File(field, field, value, mimetype)

            yield item



from ftw.blueprints.sections import mapper
from ftw.blueprints.tests.base import BlueprintTestCase


INPUT = {
    '_path': '/foo/bar',
    '_type': 'Folder',
    '_id': 'bar',
    'title': 'test',
    }


class TestTypeFieldMapper(BlueprintTestCase):

    def setUp(self):
        self.klass = mapper.TypeFieldMapper
        self.input_data = INPUT

    def _get_options(self, additional_options=None):
        options = {'blueprint': 'ftw.blueprints.typefieldmapper'}
        if additional_options:
            options.update(additional_options)
        return options

    def test_map_value(self):
        expected = self._get_expected({'_type': 'QuxFolder'})

        options = self._get_options({
            'mapping': "python: {'Folder':  ('QuxFolder', {}),}",
        })
        self.assert_result(options, expected)

    def test_map_attribute(self):
        expected = INPUT.copy()
        expected['_type'] = 'QuxFolder'
        expected['ho'] = expected['title']
        del expected['title']

        options = self._get_options({
            'mapping': "python: {'Folder':  ('QuxFolder', {'title': 'ho'}),}",
        })
        self.assert_result(options, [expected])



from ftw.blueprints.sections.multilingual import LinguaPloneItemLinker
from ftw.blueprints.testing import BLUEPRINT_FUNCTIONAL_TESTING
from ftw.blueprints.tests.base import BlueprintTestCase
from ftw.blueprints.tests.utils import TestTransmogrifier
from ftw.builder import Builder
from ftw.builder import create
from plone.app.testing import applyProfile
from plone.app.testing.helpers import setRoles
from plone.app.testing.interfaces import TEST_USER_ID
from plone.multilingual.interfaces import ITranslationManager
from plone.multilingual.interfaces import ILanguage
from copy import deepcopy


ITEMS = [{
    '_path': '/en',
    '_canonicalTranslation': False,
    '_translationOf': '/de',
    'language': 'en',
    }, {
    '_path': '/de',
    '_canonicalTranslation': True,
    '_translationOf': '/de',
    'language': 'de',
    }, {
    '_path': '/en/file',
    '_canonicalTranslation': False,
    '_translationOf': '/de/file',
    'language': 'en',
    }, {
    '_path': '/de/file',
    '_canonicalTranslation': True,
    '_translationOf': '/de/file',
    'language': 'de',
    },
]


class TestMultilingual(BlueprintTestCase):

    layer = BLUEPRINT_FUNCTIONAL_TESTING
    klass = LinguaPloneItemLinker

    def setUp(self):
        super(TestMultilingual, self).setUp()
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

        applyProfile(self.portal, 'plone.app.multilingual:default')

        self.items = deepcopy(ITEMS)
        self._setup_content()

    def _setup_content(self):
        self.folder_de = create(Builder('folder').titled('de'))
        self.folder_en = create(Builder('folder').titled('en'))

        self.file_de = create(Builder('file')
                              .within(self.folder_de)
                              .with_dummy_content())
        self.file_en = create(Builder('file')
                              .within(self.folder_en)
                              .with_dummy_content())

    def _run_transmogrifier(self):
        transmogrifier = TestTransmogrifier()
        transmogrifier.context = self.portal
        options = {'blueprint':
                   'ftw.blueprints.linguaploneitemtranslationlinker'}
        source = self.klass(transmogrifier, 'test', options, self.items)
        return list(source)

    def test_duplicate_translations_are_discarded_when_linking(self):
        folder_uhoh = create(Builder('folder').titled('uhoh'))

        self.items = self.items[:2]
        self.items.append({
            '_path': '/uhoh',
            '_canonicalTranslation': False,
            '_translationOf': '/de',
            'language': 'en',
        })
        items = self._run_transmogrifier()
        self.assertEqual(3, len(items))

        uhoh_manager = ITranslationManager(folder_uhoh)
        manager = ITranslationManager(self.folder_en)
        uhoh_manager.update()
        manager.update()

        self.assertEqual('en', ILanguage(folder_uhoh).get_language())
        self.assertEqual(1, len(uhoh_manager.get_translated_languages()))

        languages = manager.get_translated_languages()
        self.assertEqual(2, len(languages))
        self.assertIn('de', languages)
        self.assertIn('en', languages)

    def test_language_is_set(self):
        items = self._run_transmogrifier()
        self.assertEqual(4, len(items))

        self.assertEqual('de', ILanguage(self.folder_de).get_language())
        self.assertEqual('en', ILanguage(self.folder_en).get_language())

        self.assertEqual('de', ILanguage(self.file_de).get_language())
        self.assertEqual('en', ILanguage(self.file_en).get_language())

    def test_content_is_linked(self):
        items = self._run_transmogrifier()
        self.assertEqual(4, len(items))

        manager = ITranslationManager(self.folder_en)
        self.assertEqual(self.folder_de, manager.get_translation('de'),
                          'English and German content should be linked.')
        self.assertEqual(2, len(manager.get_translations()))

        manager = ITranslationManager(self.file_de)
        self.assertEqual(self.file_en, manager.get_translation('en'),
                          'English and German content should be linked.')
        self.assertEqual(2, len(manager.get_translations()))



from ftw.blueprints.interfaces import IXMLHandler
from zope.interface import implements
from xml.dom import minidom


class XMLHandler(object):
    implements(IXMLHandler)

    def parse_xml_string(self, xml_string):

        if isinstance(xml_string, unicode):
            xml_string = xml_string.encode('utf-8')

        return minidom.parseString(xml_string)

    def get_elements(self, xml, name):
        return xml.getElementsByTagName(name)

    def get_first_element(self, xml, name):
        elements = self.get_elements(xml, name)

        if elements:
            return elements[0]
        return None

    def get_element_value(self, xml, element_name):

        element = self.get_first_element(xml, element_name)

        if not element:
            return ''

        children = element.childNodes

        if not children:
            return ''

        return children[0].data

    def get_element_attribute_value(self, xml, element_name, attribute_name):

        element = self.get_first_element(xml, element_name)

        if not element:
            return ''

        return element.getAttribute(attribute_name)



from collective.transmogrifier.interfaces import ISection
from collective.transmogrifier.interfaces import ISectionBlueprint
from zope.annotation.interfaces import IAnnotations
from zope.interface import classProvides, implements


MIGR_KEY = "migr_has_default_view_object"


class AnnotateDefaultViewPathObjects(object):
    classProvides(ISectionBlueprint)
    implements(ISection)

    def __init__(self, transmogrifier, name, options, previous):
        self.previous = previous
        self.context = transmogrifier.context

    def __iter__(self):
        for item in self.previous:

            properties = item['_properties']
            for prop in properties:

                if not prop[0] == 'default_page':
                    yield item
                    continue

                obj = self.context.unrestrictedTraverse(
                    str(item['_path']).lstrip('/'), None)

                # Hack to know in an other migration if the object is
                # a default view object.
                IAnnotations(obj)[MIGR_KEY] = '%s/%s' % (
                    item['_path'], prop[1])

            yield item


class CheckIsDefaultViewObject(object):
    classProvides(ISectionBlueprint)
    implements(ISection)

    def __init__(self, transmogrifier, name, options, previous):
        self.previous = previous
        self.context = transmogrifier.context

    def __iter__(self):
        for item in self.previous:

            parent_path = item['_path'].lstrip('/').split('/')[:-1]

            if not parent_path:
                yield item
                continue

            parent = self.context.unrestrictedTraverse(
                str('/'.join(parent_path)), None)

            object_path = IAnnotations(parent).get(MIGR_KEY)

            if not object_path:
                yield item
                continue

            if object_path == item['_path']:
                item['is_default_view_object'] = True

            yield item


class UpdateDefaultViewObjectPath(object):
    classProvides(ISectionBlueprint)
    implements(ISection)

    def __init__(self, transmogrifier, name, options, previous):
        self.previous = previous

    def __iter__(self):
        for item in self.previous:
            if item.get('is_default_view_object', False):
                path = '/'.join(item['_path'].split('/')[:-1])
                item['_path'] = path

            yield item



from ftw.blueprints.sections.workflow import map_workflow
from unittest2 import TestCase
from DateTime import DateTime


WORKFLOW_HISTORY = {'test-workflow':[
    {'action': 'submit',
     'actor': 'james.bond',
     'comments': 'test-entry',
     'review_state': 'published',
     'time': DateTime(), },
    {'action': 'reject',
     'actor': 'bruce.willis',
     'comments': '',
     'review_state': 'pending',
     'time': DateTime(), }
     ]}


class TestMapWorkflow(TestCase):

    def test_input_and_output_workflows_are_equal_when_no_mapping_is_given(self):
        workflow = map_workflow(
            'test-workflow', 'test-workflow', WORKFLOW_HISTORY)

        self.assertEquals(workflow, WORKFLOW_HISTORY)

    def test_maps_history_to_the_new_workflow_id(self):
        workflow = map_workflow(
            'test-workflow', 'other-workflow', WORKFLOW_HISTORY)

        self.assertEquals(
            workflow,
            {'other-workflow': WORKFLOW_HISTORY.get('test-workflow')})

    def test_all_states_are_mapped_with_the_given_state_map(self):
        workflow = map_workflow(
            'test-workflow', 'test-workflow', WORKFLOW_HISTORY, state_map={
                'pending':'waiting',
                'published':'free',
            })

        self.assertEquals(
            [transition.get('review_state') for transition in \
                workflow.get('test-workflow')],
            ['free', 'waiting'])

    def test_all_transitions_are_mapped_with_the_given_transition_map(self):
        workflow = map_workflow(
            'test-workflow', 'test-workflow', WORKFLOW_HISTORY, transition_map={
                'submit':'send',
                'reject':'delete',
            })

        self.assertEquals(
            [transition.get('action') for transition in \
                workflow.get('test-workflow')],
            ['send', 'delete'])

    def test_returns_only_last_history_item_when_updating_history_is_deactivated(self):
        workflow = map_workflow(
            'test-workflow', 'test-workflow',
            WORKFLOW_HISTORY,
            update_history=False)

        self.assertEquals(
            workflow,
            {'test-workflow': [WORKFLOW_HISTORY.get('test-workflow')[-1]]})

    def test_no_workflow_whith_the_given_workflow_id_found(self):
        workflow = map_workflow(
            'no-workflow', 'no-workflow',
            WORKFLOW_HISTORY, )

        self.assertEquals(workflow, None)



from ftw.blueprints.sections import encapsulator
from ftw.blueprints.tests.base import BlueprintTestCase
from ftw.blueprints.tests.utils import TestTransmogrifier


INPUT = {
    '_path': '/bar',
    '_type': 'TextBlock',
    '_id': 'bar',
    'text': 'foo',
    '_content_type_text': 'text/plain'
    }


class TestEncapsulator(BlueprintTestCase):

    def setUp(self):
        self.klass = encapsulator.UnicodeAwareMimeEncapsulator
        self.options = {
            'blueprint': 'ftw.blueprints.unicodeawaremimeencapsulator',
            'data-key': 'text',
            'mimetype': "python:item.get('_content_type_%s' % key)",
            'field': 'key',
        }
        self.input_data = INPUT.copy()

    def test_str_encapsulating(self):
        source = self.klass(TestTransmogrifier(), 'test',
                            self.options, [self.input_data])
        output = iter(source).next()
        self.assertEqual('text/plain', output['text'].getContentType())

    def test_unicode_encapsulating(self):
        self.input_data['text'] = u'foo'

        source = self.klass(TestTransmogrifier(), 'test',
                            self.options, [self.input_data])
        output = iter(source).next()
        self.assertEqual('text/plain', output['text'].getContentType())



from collective.transmogrifier.interfaces import ISection
from collective.transmogrifier.interfaces import ISectionBlueprint
from collective.transmogrifier.utils import Condition
from Products.Archetypes.interfaces import IBaseObject
from zope.interface import classProvides, implements
import base64
import logging
import os


class DataUpdater(object):
    classProvides(ISectionBlueprint)
    implements(ISection)
    """
    Updates base64 encoded blob data on the item
    """

    def __init__(self, transmogrifier, name, options, previous):
        self.previous = previous
        self.context = transmogrifier.context
        self.logger = logging.getLogger(options['blueprint'])
        self.data_field = options.get('data-field')
        self.schema_field = options.get('schema-field')
        self.condition = Condition(options.get('condition', 'python:True'),
            transmogrifier, name, options)

    def __iter__(self):
        for item in self.previous:

            if not self.condition(item):
                yield item
                continue

            file_obj = self.context.unrestrictedTraverse(
                            str(item['_path']).lstrip('/'), None)

            if not file_obj and not IBaseObject.providedBy(file_obj):
                self.logger.warn(
                    "Context does not exist at %s" % item['_path'])

                yield item
                continue

            field = file_obj.Schema().get(self.schema_field)

            if not field:
                yield item
                continue

            value = item.get(self.data_field, {})

            self._update_data_field(file_obj, field, value)

            yield item

    def _update_data_field(self, obj, field, value):

        if not value.get('data'):
            return

        filename = value.get('filename')
        if not filename:
            filename = obj.getId()

        field.set(obj, base64.b64decode(value.get('data')))
        field.get(obj).filename = filename
        field.get(obj).setContentType(self.guess_mimetype(
                filename, value.get('content_type')))

        self.logger.info('Updated file data with filename: %s' % filename)

    def guess_mimetype(self, filename, default):
        # Try to guess mime type and content type
        basename, extension = os.path.splitext(filename)
        # we need only the extension without delimiter
        extension = extension[1:]
        return self.context.mimetypes_registry.extensions.get(
            extension.lower(), default)



from ftw.blueprints.handlers import XMLHandler
from ftw.blueprints.interfaces import IXMLHandler
from unittest2 import TestCase
from xml.dom import minidom
from xml.parsers.expat import ExpatError
from zope.interface.verify import verifyClass


TEST_XML = minidom.parseString("""
        <xml>
            <james>
                <bond title='\xc3\xa4gent'>007</bond>
            </james>
            <asterix>
                <bond title='hero'>Other \xc3\xa4gent</bond>
            </asterix>
        </xml>
    """)


class TestXMLHandlerClass(TestCase):

    def test_class_implements_ixmlhandler_interface(self):
        self.assertTrue(IXMLHandler.implementedBy(XMLHandler),
                        'Class %s does not implement %s.' % (
                            str(XMLHandler), str(IXMLHandler)))

        verifyClass(IXMLHandler, XMLHandler)


class TestParseXMLString(TestCase):

    def setUp(self):
        self.handler = XMLHandler()

    def test_returns_minidom_object(self):
        xml = self.handler.parse_xml_string('<james></james>')
        self.assertEquals(xml.__class__.__name__, 'Document')

    def test_functionality_with_unicode(self):
        xml = self.handler.parse_xml_string(u'<j\xe4mes></j\xe4mes>')

        self.assertIn(u'j\xe4mes', xml.firstChild.tagName)

    def test_functionality_with_utf_8(self):

        xml = self.handler.parse_xml_string(
            '<j\xc3\xa4mes></j\xc3\xa4mes>')

        self.assertIn(u'j\xe4mes', xml.firstChild.tagName)

    def test_error_message_with_invalid_xml_string(self):
        with self.assertRaises(ExpatError) as err:
            self.handler.parse_xml_string('bad_xml')
        self.assertIn('syntax error', str(err.exception))


class TestGetElements(TestCase):

    def setUp(self):
        self.handler = XMLHandler()

    def test_get_two_bond_elements_as_list(self):
        elements = self.handler.get_elements(TEST_XML, 'bond')
        self.assertIs(2, len(elements))

    def test_get_no_elements_in_empty_list(self):
        elements = self.handler.get_elements(TEST_XML, 'austin')
        self.assertIs(0, len(elements))


class TestGetFirstElement(TestCase):

    def setUp(self):
        self.handler = XMLHandler()

    def test_get_one_bond_element(self):
        element = self.handler.get_first_element(TEST_XML, 'bond')
        self.assertEquals('bond', element.tagName)

    def test_get_no_elements(self):
        element = self.handler.get_first_element(TEST_XML, 'austin')
        self.assertIs(None, element)


class TestGetElementValue(TestCase):

    def setUp(self):
        self.handler = XMLHandler()

    def test_get_007_when_asking_for_bond_tag(self):
        value = self.handler.get_element_value(TEST_XML, 'bond')
        self.assertEquals('007', value)

    def test_get_other_agent_when_asking_for_bond_tag_with_specified_xml(self):
        xml = self.handler.get_first_element(TEST_XML, 'asterix')
        value = self.handler.get_element_value(xml, 'bond')
        self.assertEquals(u'Other \xe4gent', value)

    def test_empty_string_when_no_elements_with_value(self):
        value = self.handler.get_element_value(TEST_XML, 'austin')
        self.assertEquals('', value)


class TestGetElementAttribute(TestCase):

    def setUp(self):
        self.handler = XMLHandler()

    def test_get_agent_when_asking_for_bond_title(self):
        value = self.handler.get_element_attribute_value(
            TEST_XML, 'bond', 'title')

        self.assertEquals(u'\xe4gent', value)

    def test_get_hero_when_asking_for_bond_tag_with_specified_xml(self):
        xml = self.handler.get_first_element(TEST_XML, 'asterix')
        value = self.handler.get_element_attribute_value(xml, 'bond', 'title')

        self.assertEquals('hero', value)

    def test_empty_string_when_attribute_doesnt_exist(self):
        value = self.handler.get_element_attribute_value(
            TEST_XML, 'bond', 'nothing')
        self.assertEquals('', value)

    def test_empty_string_when_element_doenst_exist(self):
        value = self.handler.get_element_attribute_value(
            TEST_XML, 'austin', 'title')
        self.assertEquals('', value)



from ftw.blueprints.sections.portlet import PortletHandler
from unittest2 import TestCase


class DummyAssignment(object):

    def __init__(self, **kwargs):
        self.kwargs = kwargs


DUMMY_ASSIGNMENT_PATH = 'ftw.blueprints.tests.test_portlet.DummyAssignment'


class TestPortletHandlerAssignmentObject(TestCase):

    def get_handler(self, path=DUMMY_ASSIGNMENT_PATH):
        return PortletHandler(
            'manager_name',
            path,
            'portlet_id')

    def test_get_instance_of_dummyassignment_class(self):
        handler = self.get_handler()
        assignment_obj = handler.get_assignment_object()
        self.assertEquals(DummyAssignment, assignment_obj.__class__)

    def test_get_dummyassignment_without_properties(self):
        handler = self.get_handler()
        assignment_obj = handler.get_assignment_object()
        self.assertEquals(assignment_obj.kwargs, {})

    def test_get_dummyassignent_with_a_property(self):
        handler = self.get_handler()
        assignment_obj = handler.get_assignment_object(dict(title='Test'))
        self.assertEquals(assignment_obj.kwargs, {'title': 'Test'})

    def test_raise_importerror_if_the_path_to_assignment_class_does_not_exist(self):
        handler = self.get_handler('ftw.blueprints.not-existing.path')

        with self.assertRaises(ImportError) as err:
            handler.get_assignment_object()

        self.assertIn('not-existing', str(err.exception))

    def test_raise_attributeerror_if_the_assignment_class_does_not_exist(self):
        handler = self.get_handler(
            'ftw.blueprints.tests.test_portlet.not-existing-class')

        with self.assertRaises(AttributeError) as err:
            handler.get_assignment_object()

        self.assertIn('not-existing-class', str(err.exception))



from collective.transmogrifier.interfaces import ISectionBlueprint, ISection
from collective.transmogrifier.utils import Expression
from zope.interface import classProvides, implements
import logging


class Logger(object):
    classProvides(ISectionBlueprint)
    implements(ISection)

    def __init__(self, transmogrifier, name, options, previous):
        self.previous = previous
        self.logger = logging.getLogger(options['blueprint'])
        self.print_out = Expression(
            options.get('print_out'), transmogrifier, name, options)

    def __iter__(self):
        for item in self.previous:
            self.logger.info(str(self.print_out(item)))
            yield item
            continue



from Testing.ZopeTestCase.utils import setupCoreSessions
from ftw.builder.testing import BUILDER_LAYER
from ftw.builder.testing import functional_session_factory
from ftw.builder.testing import set_builder_session_factory
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer
from zope.configuration import xmlconfig


class BlueprintLayer(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE, BUILDER_LAYER)

    def setUpZope(self, app, configurationContext):
        import ftw.blueprints
        xmlconfig.file('configure.zcml', ftw.blueprints,
                       context=configurationContext)

        import plone.dexterity
        xmlconfig.file('configure.zcml', plone.dexterity,
                       context=configurationContext)

        import plone.app.dexterity
        xmlconfig.file('configure.zcml', plone.app.dexterity,
                       context=configurationContext)

        import plone.app.multilingual
        xmlconfig.file('configure.zcml', plone.app.multilingual,
                       context=configurationContext)

        setupCoreSessions(app)


BLUEPRINT_FIXTURE = BlueprintLayer()
BLUEPRINT_INTEGRATION_TESTING = IntegrationTesting(
    bases=(BLUEPRINT_FIXTURE, ), name="ftw.blueprints:Integration")
BLUEPRINT_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(BLUEPRINT_FIXTURE,
           set_builder_session_factory(functional_session_factory)),
    name="ftw.blueprints:Functional")



from collective.transmogrifier.interfaces import ISection
from collective.transmogrifier.interfaces import ISectionBlueprint
from collective.transmogrifier.utils import defaultMatcher
from plone.multilingual.interfaces import ILanguage
from plone.multilingual.interfaces import IMutableTG
from plone.multilingual.interfaces import ITranslatable
from plone.multilingual.interfaces import ITranslationManager
from plone.uuid.interfaces import IUUIDGenerator
from zope.component import getUtility
from zope.interface import classProvides
from zope.interface import implements


class LinguaPloneItemLinker(object):
    """Link content from a page that was translated with LinguaPlone.
    The items in the pipeline are ecpected to have the following keys:

        - a boolean indicating whether they are a 'canonical' translation
        - a reference (path) to the canonical translation (points to the item
          itself when the item is a canonical translation)
        - a path

    This section expects that plone content has already been constructed. The
    new translations are then created with plone.app.multilingual.

    """
    classProvides(ISectionBlueprint)
    implements(ISection)

    def __init__(self, transmogrifier, name, options, previous):
        self.previous = previous
        self.context = transmogrifier.context
        self.pathkey = defaultMatcher(options, 'path-key', name, 'path')
        self.canonicalkey = defaultMatcher(options, 'canonical-key', name,
                                           'canonicalTranslation')
        self.translationkey = defaultMatcher(options, 'translation-key', name,
                                             'translationOf')
        self.uuid_generator = getUtility(IUUIDGenerator)

        self.deferred = []

    def _traverse(self, path):
        return self.context.unrestrictedTraverse(path.lstrip('/'), None)

    def __iter__(self):
        for item in self.previous:
            pathkey = self.pathkey(*item.keys())[0]
            if not pathkey:
                yield item
                continue

            path = item[pathkey]
            if isinstance(path, unicode):
                path = path.encode('ascii')

            obj = self._traverse(path)
            if obj is None:
                yield item
                continue

            canonicalkey = self.canonicalkey(*item.keys())[0]
            translationkey = self.translationkey(*item.keys())[0]

            if not canonicalkey:
                yield item
                continue

            canonicalpath = item[translationkey]
            language = item['language']
            if ITranslatable.providedBy(obj):
                ILanguage(obj).set_language(language)
            if item[canonicalkey]:
                IMutableTG(obj).set(self.uuid_generator())
                manager = ITranslationManager(obj)
                manager.register_translation(language, obj)
            else:
                self.deferred.append((path, canonicalpath, language))
            yield item

        self._update_deferred()

    def _update_deferred(self):
        for path, canonicalpath, language in self.deferred:
            obj = self._traverse(path)
            if obj is None:
                continue
            canonical = self._traverse(canonicalpath)
            if canonical is None:
                continue

            if (ITranslatable.providedBy(obj) and
                    ITranslatable.providedBy(canonical)):
                try:
                    translation_group = IMutableTG(canonical).get()
                    IMutableTG(obj).set(translation_group)
                    manager = ITranslationManager(obj)
                    manager.register_translation(language, obj)
                except KeyError:
                    # put obj in a separate translation group when registration
                    # fails
                    IMutableTG(obj).set(self.uuid_generator())



from Acquisition import aq_inner
from Acquisition import aq_parent
from collective.transmogrifier.interfaces import ISection
from collective.transmogrifier.interfaces import ISectionBlueprint
from collective.transmogrifier.utils import defaultMatcher
from operator import itemgetter
from zope.annotation.interfaces import IAnnotations
from zope.interface import classProvides, implements


ANNOTATION_KEY = 'ftw.blueprints-position'


class PositionInParentUpdater(object):
    classProvides(ISectionBlueprint)
    implements(ISection)

    def __init__(self, transmogrifier, name, options, previous):
        self.previous = previous
        self.context = transmogrifier.context
        self.pathkey = defaultMatcher(options, 'path-key', name, 'path')
        self.positionkey = defaultMatcher(options, 'position-key', name, 'gopip')

    def __iter__(self):
        for item in self.previous:
            keys = item.keys()
            pathkey = self.pathkey(*keys)[0]
            positionkey = self.positionkey(*keys)[0]
            path = item[pathkey]
            position = item.get(positionkey)

            obj = self.context.unrestrictedTraverse(
                str(path).lstrip('/'), None)
            if obj is not None:
                self.updateObjectPosition(obj, position)

            yield item

    def updateObjectPosition(self, obj, position):
        """Store the position we want on the object and order all children
        of the parent according to their stored positions.

        This allows to partially migrate different types in a folder in different
        steps and will reorder already migrated siblings correctly because we
        have stored the position from the source installation.
        """

        if position is not None:
            self.store_position_for_obj(obj, position)
        parent = aq_parent(aq_inner(obj))
        self.reorder_children(parent)

    def store_position_for_obj(self, obj, position):
        IAnnotations(obj)[ANNOTATION_KEY] = position

    def reorder_children(self, obj):
        ordered_sibling_ids = self.get_ordered_children_ids_from_annotations(obj)
        obj.moveObjectsByDelta(ordered_sibling_ids, - len(ordered_sibling_ids))

    def get_ordered_children_ids_from_annotations(self, container):
        def get_position_from_annotations(item):
            id_, obj = item
            try:
                return IAnnotations(obj)[ANNOTATION_KEY]
            except (TypeError, KeyError):
                return 10000

        ordered_children = sorted(container.objectItems(), key=get_position_from_annotations)
        return map(itemgetter(0), ordered_children)



from ftw.blueprints.sections import inserter
from ftw.blueprints.tests.base import BlueprintTestCase


INPUT = {
    '_path': '/foo/bar',
    '_type': 'Folder',
    '_id': 'bar',
    'title': 'test',
    }


class ObjectInserter(BlueprintTestCase):

    def setUp(self):
        self.klass = inserter.AdditionalObjectInserter
        self.input_data = INPUT

    def test_insert_object_at_a_given_path(self):

        options = {
            'content-type': 'Page',
            'additional-id': 'string:item',
            'new-path': 'python:"/foo/given/path"'
            }

        expected = [
            {'_interfaces': [],
             '_type': 'Page',
             '_path': '/foo/given/path',
             '_id': 'item',
             '_annotations': {}},
             INPUT,
        ]

        self.assert_result(options, expected)


class TestChildInserter(BlueprintTestCase):

    def setUp(self):
        self.klass = inserter.ChildInserter
        self.input_data = INPUT

    def test_blueprint_with_default_settings(self):
        expected = [
            INPUT,
            {'_interfaces': [],
             '_type': 'Page',
             '_path': '/foo/bar/item',
             '_id': 'item',
             '_annotations': {}},
        ]

        self.assert_result(get_options(), expected)

    def test_blueprint_with_condition_false(self):
        expected = [
            INPUT,
        ]

        self.assert_result(get_options(
            condition='python:False'), expected)

    def test_blueprint_with_additional_interfaces(self):
        expected = [
            INPUT,
            {'_interfaces': ["ITest1", "ITest2"],
             '_type': 'Page',
             '_path': '/foo/bar/item',
             '_id': 'item',
             '_annotations': {}},
        ]

        self.assert_result(get_options(
            interfaces='python:["ITest1", "ITest2"]'), expected)

    def test_blueprint_with_additional_annotations(self):
        expected = [
            INPUT,
            {'_interfaces': [],
             '_type': 'Page',
             '_path': '/foo/bar/item',
             '_id': 'item',
             '_annotations': {"viewname": "portlet"}},
        ]

        self.assert_result(get_options(
                annotations='python:{"viewname": "portlet"}'), expected)

    def test_blueprint_with_additional_metadata(self):
        expected = [
            INPUT,
            {'_interfaces': [],
             '_type': 'Page',
             '_path': '/foo/bar/item',
             '_annotations': {},
             '_id': 'item',
             'title': 'bar'},
        ]

        self.assert_result(get_options(
            metadata='python:{"title": lambda item: item["_id"]}'), expected)


class TestParentInserter(BlueprintTestCase):

    def setUp(self):
        self.klass = inserter.ParentInserter
        self.input_data = INPUT

    def test_blueprint_with_default_settings(self):
        expected = [
            {'_interfaces': [],
             '_path': '/foo/item',
             '_type': 'Page',
             '_id': 'item',
             '_annotations': {}},
            self.get_expected_output(),
        ]

        self.assert_result(get_options(), expected)

    def test_blueprint_with_condition_false(self):
        expected = [
            INPUT,
        ]

        self.assert_result(get_options(
            condition='python:False'), expected)

    def test_blueprint_with_additional_interfaces(self):
        expected = [
            {'_interfaces': ["ITest1", "ITest2"],
             '_path': '/foo/item',
             '_type': 'Page',
             '_id': 'item',
             '_annotations': {}},
            self.get_expected_output(),
        ]

        self.assert_result(get_options(
            interfaces='python:["ITest1", "ITest2"]'), expected)

    def test_blueprint_with_additional_annotations(self):
        expected = [
            {'_interfaces': [],
             '_type': 'Page',
             '_path': '/foo/item',
             '_id': 'item',
             '_annotations': {"viewname": "portlet"}},
            self.get_expected_output(),
        ]

        self.assert_result(get_options(
                annotations='python:{"viewname": "portlet"}'), expected)

    def test_blueprint_with_additional_metadata(self):
        expected = [
            {'_interfaces': [],
             '_type': 'Page',
             '_path': '/foo/item',
             '_annotations': {},
             '_id': 'item',
             'title': 'bar'},
            self.get_expected_output(),
        ]

        self.assert_result(get_options(
            metadata='python:{"title": lambda item: item["_id"]}'), expected)

    def get_expected_output(self):
        input_ = INPUT.copy()
        input_['_path'] = '/foo/item/bar'
        return input_


def get_options(
    content_type="Page",
    additional_id="string:item",
    condition=None,
    interfaces=None,
    annotations=None,
    metadata=None):

    options = {
        'content-type': content_type,
        'additional-id': additional_id,
        }

    if condition:
        options.update({'condition': condition})

    if interfaces:
        options.update({'_interfaces': interfaces})

    if annotations:
        options.update({'_annotations': annotations})

    if metadata:
        options.update({'metadata-key': metadata})

    return options



from collective.transmogrifier.interfaces import ISection
from collective.transmogrifier.interfaces import ISectionBlueprint
from unittest2 import TestCase
from zope.interface.verify import verifyClass
from zope.interface.verify import verifyObject
from ftw.blueprints.tests.utils import TestTransmogrifier


class BlueprintTestCase(TestCase):

    klass = None
    input_data = None

    def _get_expected(self, changes=None):
        expected = self.input_data.copy()
        if changes:
            expected.update(changes)
        return [expected]

    def test_implements_interface(self):
        # skip if not inherited
        if self.__class__ == BlueprintTestCase:
            return

        self.assertIsNotNone(self.klass, 'please configure a klass')
        self.assert_class_implements(self.klass, ISection)
        self.assert_class_provides(self.klass, ISectionBlueprint)

    def assert_class_implements(self, klass, interface):
        self.assertTrue(interface.implementedBy(klass),
                        'Class %s does not implement %s.' % (
                            str(klass), str(interface)))

        verifyClass(interface, klass)

    def assert_class_provides(self, klass, interface):
        self.assertTrue(interface.providedBy(klass),
                        'Class %s does not provide %s.' % (
                            str(klass), str(interface)))

        verifyObject(interface, klass)

    def assert_result(self, options, expected, input_data=None, klass=None):
        inserter = klass or self.klass
        input_data = input_data or self.input_data
        input_data = [input_data.copy()]
        source = inserter(TestTransmogrifier(), 'test', options, input_data)
        output = list(source)

        self.maxDiff = None
        self.assertEqual(output, expected)



from collective.transmogrifier.interfaces import ISection
from collective.transmogrifier.interfaces import ISectionBlueprint
from collective.transmogrifier.utils import Condition
from collective.transmogrifier.utils import Expression
from collective.transmogrifier.utils import traverse
from DateTime import DateTime
from Products.Archetypes.interfaces import IBaseObject
from Products.CMFCore.utils import getToolByName
from zope.interface import classProvides, implements


def map_workflow(old_workflow_id, new_workflow_id,
    workflow_history,
    state_map={},
    transition_map={},
    update_history=True):

    old_workflow = workflow_history.get(old_workflow_id, [])

    if not old_workflow:
        return None

    if not update_history:
        # The last element of the wf-history contains the current
        # review-state informations.
        old_workflow = [old_workflow[-1]]

    new_workflow = {new_workflow_id: []}
    for transition in old_workflow:

        action = transition_map.get(
            transition.get('action'), transition.get('action'))

        review_state = state_map.get(
            transition.get('review_state'),
            transition.get('review_state'))

        new_workflow[new_workflow_id].append(
            dict(
                action=action,
                actor=transition.get('actor'),
                comments=transition.get('comments'),
                review_state=review_state,
                time=DateTime(transition.get('time'))
            )
        )

    return new_workflow


class WorkflowManager(object):
    classProvides(ISectionBlueprint)
    implements(ISection)

    def __init__(self, transmogrifier, name, options, previous):
        self.previous = previous
        self.context = transmogrifier.context
        self.wftool = getToolByName(self.context, 'portal_workflow')

        self.pathkey = options.get('path-key', '_path')
        self.history_key = options.get('history-key', '_workflow_history')
        self.state_key = options.get('state-key', '_review_state')

        self.update_history = options.get('update-history', True)

        self.old_workflow_id = options.get('old-workflow-id')
        self.new_workflow_id = options.get(
            'new-workflow-id', self.old_workflow_id)

        self.state_map = Expression(
            options.get('state-map', 'python:{}'),
            transmogrifier, name, options)
        self.transition_map = Expression(
            options.get('transition-map', 'python:{}'),
            transmogrifier, name, options)

        self.condition = Condition(options.get('condition', 'python:True'),
            transmogrifier, name, options)

    def __iter__(self):
        for item in self.previous:
            if not self.condition(item):
                yield item
                continue

            obj = traverse(self.context, item.get(self.pathkey, ''))
            if not obj or not IBaseObject.providedBy(obj) or not hasattr(
                obj, 'workflow_history'):
                yield item
                continue

            new_workflow_history = map_workflow(
                self.old_workflow_id,
                self.new_workflow_id,
                item.get(self.history_key, {}),
                self.state_map(item),
                self.transition_map(item),
                self.update_history)

            if new_workflow_history:
                self.set_workflow_on_obj(obj, new_workflow_history)

            yield item

    def set_workflow_on_obj(self, obj, workflow_history):
        obj.workflow_history.clear()
        obj.workflow_history.update(workflow_history)
        workflows = self.wftool.getWorkflowsFor(obj)
        if workflows:
            workflows[0].updateRoleMappingsFor(obj)



from setuptools import setup, find_packages
import os

version = '1.0.0'

tests_require = [
    'ftw.builder',
    'ftw.testing',
    'plone.app.dexterity',
    'plone.app.multilingual',
    'plone.app.testing',
    'plone.multilingualbehavior',
    'plone.testing',
    'unittest2',
    'zope.configuration',
    ]


setup(name='ftw.blueprints',
      version=version,
      description="Provides useful blueprints for migrations with " + \
          "transmogrifier",
      long_description=open('README.rst').read() + '\n' + \
          open(os.path.join('docs', 'HISTORY.txt')).read(),

      classifiers=[
        'Framework :: Plone',
        'Framework :: Plone :: 4.2',
        'Framework :: Plone :: 4.3',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries :: Python Modules',
        ],
      keywords='',
      author='4teamwork AG',
      author_email='mailto:info@4teamwork.ch',
      url='https://github.com/4teamwork/ftw.blueprints',
      license='GPL2',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['ftw'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'collective.jsonmigrator',
          'collective.blueprint.jsonmigrator',
          'collective.transmogrifier >= 1.5',
          'transmogrify.dexterity',
          'ftw.inflator',
          # -*- Extra requirements: -*-
      ],

      tests_require=tests_require,
      extras_require={'tests': tests_require},

      entry_points="""
      # -*- Entry points: -*-
      [z3c.autoinclude.plugin]
      target = plone
      """,
      )



from collective.transmogrifier.interfaces import ISection
from collective.transmogrifier.interfaces import ISectionBlueprint
from DateTime import DateTime
from ftw.blueprints.handlers import XMLHandler
from ftw.blueprints.interfaces import IFormGenField
from ftw.blueprints.interfaces import IPFM2PFGConverter
from plone.i18n.normalizer.interfaces import IIDNormalizer
from zope.component import getUtility
from zope.component import queryUtility
from zope.interface import classProvides, implements


class PFM2PFGConverter(object):
    implements(IPFM2PFGConverter)

    def __init__(self, item, xml_string):

        self.handler = XMLHandler()
        self.item = item
        self.xml_string = xml_string
        self.normalizer = getUtility(IIDNormalizer)

    def __iter__(self):

        xml = self.handler.parse_xml_string(self.xml_string)

        for group in self.handler.get_elements(xml, 'group'):

            yield self.begin_group(group)

            for field in self.handler.get_elements(group, 'field'):

                yield self.get_pfg_field(field)

            yield self.end_group(group)

    def get_pfg_field(self, field):

        pfm_type = self.handler.get_element_value(field, 'type')

        field_utility = self.get_field_utility(pfm_type)
        if not field_utility:
            raise

        pfg_field = field_utility(field, self.item)
        pfg_field.fill_field()

        return pfg_field

    def get_field_utility(self, type_):
        """Returns the assosiated utility for pfg fields
        """

        return queryUtility(
            IFormGenField,
            'ftw.blueprints.pfm2pfg.%s' % type_)

    def end_group(self, xml):

        item = {}

        title = self.handler.get_element_value(xml, 'title')

        if title == 'Default':
            return item

        title = self.normalizer.normalize(title)

        item['_id'] = '%s-end' % title
        item['_type'] = 'FieldsetEnd'
        item['_path'] = '%s/%s-stop' % (self.item['_path'], title)

        return item

    def begin_group(self, xml):

        item = {}

        title = self.handler.get_element_value(xml, 'title')

        if title == 'Default':
            return item

        item['title'] = title

        title = self.normalizer.normalize(title)

        item['_id'] = '%s-start' % title
        item['_type'] = 'FieldsetStart'
        item['_path'] = '%s/%s-start' % (self.item['_path'], title)

        return item


class FormGenField(dict):
    implements(IFormGenField)

    form_type = None

    def __init__(self, field, item):

        self.handler = XMLHandler()
        self.field = field
        self.item = item

    def fill_field(self):

        for method in dir(self):
            if method.startswith('set_'):
                getattr(self, method)()

    def _get_filtered_element_value(self, name):

        value = self.handler.get_element_value(self.field, name)
        type_ = self.handler.get_element_attribute_value(
            self.field, name, 'type')

        if type_ == 'float':
            return float(value)

        elif type_ == 'int':
            if name in ['enabled', 'required', 'hidden']:
                return bool(int(value))
            return int(value)

        elif type_ == 'list':
            return eval(value)

        elif type_ == 'datetime':
            return DateTime(value)

        if isinstance(value, unicode):
            return value.encode('utf-8')
        else:
            return value

    def set_path(self):
        self['_path'] = '%s/%s' % (
            self.item['_path'], self._get_filtered_element_value('id'))

    def set_id(self):
        self['_id'] = self._get_filtered_element_value('id')

    def set_type(self):
        self['_type'] = self.form_type

    def set_title(self):
        self['title'] = self._get_filtered_element_value('title')

    def set_required(self):
        self['required'] = self._get_filtered_element_value('required')

    def set_description(self):
        self['description'] = self._get_filtered_element_value('description')

    def set_hidden(self):
        self['hidden'] = self._get_filtered_element_value('hidden')

    def set_default(self):
        self['fgDefault'] = self._get_filtered_element_value('default')


class FormStringField(FormGenField):
    classProvides(IFormGenField)

    form_type = 'FormStringField'


class FormTextField(FormGenField):
    classProvides(IFormGenField)

    form_type = 'FormTextField'


class FormPasswordField(FormGenField):
    classProvides(IFormGenField)

    form_type = 'FormPasswordField'


class FormRichLabelField(FormGenField):
    classProvides(IFormGenField)

    form_type = 'FormRichLabelField'


class FormIntegerField(FormGenField):
    classProvides(IFormGenField)

    form_type = 'FormIntegerField'


class FormFixedPointField(FormGenField):
    classProvides(IFormGenField)

    form_type = 'FormFixedPointField'


class FormDateField(FormGenField):
    classProvides(IFormGenField)

    form_type = 'FormDateField'


class FormFileField(FormGenField):
    classProvides(IFormGenField)

    form_type = 'FormFileField'


class FormLinesField(FormGenField):
    classProvides(IFormGenField)

    form_type = 'FormLinesField'


class FormBooleanField(FormGenField):
    classProvides(IFormGenField)

    form_type = 'FormBooleanField'


class FormStringFieldEmail(FormStringField):
    classProvides(IFormGenField)

    def set_fg_string_validator(self):
        self['fgStringValidator'] = 'isEmail'


class FormStringFieldURL(FormStringField):
    classProvides(IFormGenField)

    def set_fg_string_validator(self):
        self['fgStringValidator'] = 'isURL'


class FormSelectionField(FormGenField):
    classProvides(IFormGenField)

    form_type = 'FormSelectionField'

    def set_fg_vocabulary(self):
        value = self._get_filtered_element_value('items')
        self['fgVocabulary'] = [key for key, val in value]

    def set_default(self):
        value = self._get_filtered_element_value('default')

        if value:
            self['fgDefault'] = value


class FormSelectionFieldSelect(FormSelectionField):
    classProvides(IFormGenField)

    def set_fg_format(self):
        self['fgFormat'] = 'select'


class FormSelectionFieldRadio(FormSelectionField):
    classProvides(IFormGenField)

    def set_fg_format(self):
        self['fgFormat'] = 'radio'


class FormMultiSelectionField(FormSelectionField):
    classProvides(IFormGenField)

    form_type = 'FormMultiSelectionField'


class FormMultiSelectionFieldSelect(FormMultiSelectionField):
    classProvides(IFormGenField)

    def set_fg_format(self):
        self['fgFormat'] = 'select'


class FormMultiSelectionFieldCheckbox(FormMultiSelectionField):
    classProvides(IFormGenField)

    def set_fg_format(self):
        self['fgFormat'] = 'checkbox'


class FormMailerFieldsInserter(object):
    classProvides(ISectionBlueprint)
    implements(ISection)

    def __init__(self, transmogrifier, name, options, previous):
        self.context = transmogrifier.context
        self.previous = previous

    def __iter__(self):

        for item in self.previous:

            # Returns the FormFolder
            yield item

            # Cleanup the auto-created fields before adding the fields
            self.cleanup(item['_path'])

            # After cleanup, we handle the form fields
            form_data = item.get('form_data')

            if not form_data:
                yield item
                continue

            converter = PFM2PFGConverter(item, form_data)

            for field in converter:
                yield field

    def cleanup(self, path):

        to_remove = ['replyto', 'topic', 'comments']

        folder = self.context.unrestrictedTraverse(
                            str(path).lstrip('/'), None)

        if not folder:
            return

        for field in to_remove:
            if not hasattr(folder, field):
                continue

            folder._delObject(field)

        folder.reindexObject()



from zope.interface import Interface


class IXMLHandler(Interface):
    """Provides useful methods to handle xml data with minidom
    """

    def parse_xml_string(xml_string):
        """Parses a xml string with minidom
        """

    def get_elements(xml, name):
        """Return the given elements
        """

    def get_first_element(xml, name):
        """Return the first found element
        """

    def get_element_value(xml, element_name):
        """Returns the value of the given element name
        """

    def get_element_attribute_value(xml, element_name, attribute_name):
        """Returns the attribute value of the given attribute name from the
        given element name
        """


class IPFM2PFGConverter(Interface):
    """Converts a old PloneFormMailer formular into a new PloneFormGen
    """

    def __init__(item, xml_string):
        """
        """

    def __iter__():
        """Returns the FormFields as a dict
        """

    def end_group(xml):
        """End of a fieldgroup
        """

    def begin_group(xml):
        """Begin of a fieldgroup
        """

    def get_pfg_field(field):
        """Returns the PloneFormGen Field as a dict
        """


class IFormGenField(Interface):
    """PloneFormGen representation dict object
    """

    def __init__(field):
        """
        """

    def fill_field():
        """Fills the dict with its values
        """



from ftw.blueprints.sections.position import PositionInParentUpdater
from ftw.blueprints.testing import BLUEPRINT_INTEGRATION_TESTING
from ftw.blueprints.tests.base import BlueprintTestCase
from ftw.blueprints.tests.utils import TestTransmogrifier
from ftw.builder import Builder
from ftw.builder import create
from plone.app.testing.helpers import setRoles
from plone.app.testing.interfaces import TEST_USER_ID
from Products.CMFCore.utils import getToolByName


class TestPositionInParentUpdater(BlueprintTestCase):
    layer = BLUEPRINT_INTEGRATION_TESTING
    klass = PositionInParentUpdater

    def setUp(self):
        super(TestPositionInParentUpdater, self).setUp()
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

    def test_updating_object_position_in_folder(self):
        folder = create(Builder('folder').titled('folder'))
        create(Builder('folder').titled('three').within(folder))
        create(Builder('folder').titled('two').within(folder))
        create(Builder('folder').titled('one').within(folder))

        self.assert_obj_positions(folder, ['three', 'two', 'one'])
        self.assert_catalog_positions(folder, ['three', 'two', 'one'])

        self.run_transmogrifier([{'_path': 'folder/one', '_gopip': 11},
                                 {'_path': 'folder/two', '_gopip': 22},
                                 {'_path': 'folder/three', '_gopip': 33}])

        self.assert_obj_positions(folder, ['one', 'two', 'three'])
        self.assert_catalog_positions(folder, ['one', 'two', 'three'])

        self.run_transmogrifier([{'_path': 'folder/three', '_gopip': 33}])
        self.run_transmogrifier([{'_path': 'folder/two', '_gopip': 22}])
        self.run_transmogrifier([{'_path': 'folder/one', '_gopip': 11}])

        self.assert_obj_positions(folder, ['one', 'two', 'three'])
        self.assert_catalog_positions(folder, ['one', 'two', 'three'])

    def test_updating_object_position_in_plone_site(self):
        create(Builder('folder').titled('three'))
        create(Builder('folder').titled('two'))
        create(Builder('folder').titled('one'))

        self.assert_obj_positions(self.portal, ['three', 'two', 'one'])
        self.assert_catalog_positions(self.portal, ['three', 'two', 'one'])

        self.run_transmogrifier([{'_path': 'one', '_gopip': 11},
                                 {'_path': 'two', '_gopip': 22},
                                 {'_path': 'three', '_gopip': 33}])

        self.assert_obj_positions(self.portal, ['one', 'two', 'three'])
        self.assert_catalog_positions(self.portal, ['one', 'two', 'three'])

        self.run_transmogrifier([{'_path': 'three', '_gopip': 33}])
        self.run_transmogrifier([{'_path': 'two', '_gopip': 22}])
        self.run_transmogrifier([{'_path': 'one', '_gopip': 11}])

        self.assert_obj_positions(self.portal, ['one', 'two', 'three'])
        self.assert_catalog_positions(self.portal, ['one', 'two', 'three'])

    def run_transmogrifier(self, items):
        transmogrifier = TestTransmogrifier()
        transmogrifier.context = self.portal
        options = {'blueprint': 'ftw.blueprints.positionupdater'}
        source = self.klass(transmogrifier, 'test', options, items)
        return list(source)

    def assert_obj_positions(self, container, expected):
        got = [id_ for id_ in container.objectIds() if id_ in expected]
        self.assertEquals(expected, got)

    def assert_catalog_positions(self, container, expected):
        catalog = getToolByName(self.portal, 'portal_catalog')
        query = {'path': {'query': '/'.join(container.getPhysicalPath()),
                          'depth': 1},
                 'id': expected,
                 'sort_on': 'getObjPositionInParent'}
        got = [brain.getId for brain in catalog(query)]
        self.assertEquals(expected, got, 'Catalog indexes not up to date.')



from collective.transmogrifier.interfaces import ISection
from collective.transmogrifier.interfaces import ISectionBlueprint
from collective.transmogrifier.utils import Condition
from collective.transmogrifier.utils import Expression
from zope.interface import classProvides, implements
import os


class ObjectInserter(object):
    """Baseclass to add an item to the transmogrifier pipeline.
    """
    def __init__(self, transmogrifier, name, options, previous):
        self.previous = previous
        self.condition = Condition(options.get('condition', 'python:True'),
            transmogrifier, name, options)

        self.path_key = options.get('path-key', '_path')
        self.contenttype_key = options.get('contenttype-key', '_type')
        self.id_key = options.get('id-key', '_id')
        self.interfaces_key = options.get('interfaces-key', '_interfaces')
        self.annotations_key = options.get('annotations-key', '_annotations')

        self.content_type = options.get('content-type')
        self.additional_id = Expression(
            options.get('additional-id'), transmogrifier, name, options)
        self.interfaces = Expression(
            options.get(
                '_interfaces', 'python:[]'), transmogrifier, name, options)
        self.annotations = Expression(
            options.get(
                '_annotations', 'python:{}'), transmogrifier, name, options)
        self.metadata = Expression(
            options.get(
                'metadata-key', 'python:{}'), transmogrifier, name, options)
        self.path = None

    def __iter__(self):
        for item in self.previous:

            if not self.condition(item):
                yield item
                continue

            yield self.get_additional_item(item)
            yield item

    def get_additional_item(self, item):
        return self.create_additional_item(
            item,
            self.path(item),
            self.additional_id(item),
            self.content_type,
            self.interfaces(item),
            self.annotations(item))

    def create_additional_item(
        self, item, path, additional_id,
        content_type, interfaces, annotations):

        additional_item = {
                self.contenttype_key: content_type,
                self.interfaces_key: interfaces,
                self.annotations_key: annotations,
                self.id_key: additional_id,
                self.path_key: path,
                }

        self.extend_metadata(item, additional_item, self.metadata(item))

        return additional_item

    def extend_metadata(self, item, additional_item, metadata):
        """Extends the new item with additional metadata
        """

        for key, value_key in metadata.items():
            # It 's possible to map a field to a callable function
            # instead of an allready existing key
            # per example:
            # -- lambda item: item.get('title', '')
            if callable(value_key):
                additional_item[key] = value_key(item)

            elif item.get(value_key) is not None:
                additional_item[key] = item.get(value_key)


class AcquisitionInserter(ObjectInserter):
    """Baseclass to add an item to the transmogrifier pipeline as parent or
    child.
    """
    insert_as_parent = False

    def __iter__(self):
        for item in self.previous:

            if not self.condition(item):
                yield item
                continue

            additional_item = self.get_additional_item(item)

            if self.insert_as_parent:
                parent, child = additional_item, item
                self.rename_item(parent, additional_item[self.id_key])
            else:
                parent, child = item, additional_item
                self.rename_item(child, additional_item[self.id_key])

            self.move_item_into_container(child, parent)

            yield parent
            yield child

    def get_additional_item(self, item):
        return self.create_additional_item(
            item,
            item[self.path_key],
            self.additional_id(item),
            self.content_type,
            self.interfaces(item),
            self.annotations(item))

    def rename_item(self, item, new_id):
        item[self.path_key] = os.path.join(
            os.path.dirname(item[self.path_key]), new_id)

    def move_item_into_container(self, item, container):
        item_id = os.path.basename(item[self.path_key])
        item[self.path_key] = os.path.join(container[self.path_key], item_id)


class ChildInserter(AcquisitionInserter):
    """Inserts a new item to the transmogrifier pipeline as a child
    """
    classProvides(ISectionBlueprint)
    implements(ISection)


class ParentInserter(AcquisitionInserter):
    """Inserts a new item to the transmogrifier pipeline as a parent
    """
    classProvides(ISectionBlueprint)
    implements(ISection)

    insert_as_parent = True


class AdditionalObjectInserter(ObjectInserter):
    """Inserts a new item to the transmogrifier pipeline on a variable path
    """
    classProvides(ISectionBlueprint)
    implements(ISection)

    def __init__(self, transmogrifier, name, options, previous):
        super(AdditionalObjectInserter, self).__init__(
            transmogrifier, name, options, previous)

        self.path = Expression(
            options.get('new-path', None), transmogrifier, name, options)
